import sys

class Pokladnicka:
    def __init__(self, verbose = False):
        self.obmedzenia = []
        self.instrukcie = []
        self.pokladnicka = {}
        self.verbose = verbose
        self.max_krokov = 200 if verbose else 10000

    def parsuj_mnozinu(self, mnozina):
        odpoved = {}
        if mnozina.strip() == '':
            return odpoved
        separator = '+' if '+' in mnozina else ','
        tokeny = [ x.strip() for x in mnozina.split(separator) ]
        try:
            for token in tokeny:
                assert token != ''
                casti = token.split()
                assert len(casti) <= 2
                if len(casti) == 1:
                    pocet, farba = 1, casti[0]
                else:
                    pocet, farba = int(casti[0]), casti[1]
                    assert pocet > 0
                    assert pocet <= 10**9
                if farba not in odpoved:
                    odpoved[farba] = 0
                odpoved[farba] += pocet
        except:
            assert False, f'Mnozina "{mnozina}" nie je syntakticky korektna.'
        return odpoved

    def pridaj_obmedzenie(self, obmedzenie):
        try:
            lava, prava = [ x.strip() for x in obmedzenie.split('<=') ]
            lava = self.parsuj_mnozinu(lava)
            prava = int(prava)
            assert prava > 0
        except:
            assert False, f'Obmedzenie "{obmedzenie}" nie je syntakticky korektne.'
        if self.verbose:
            print( f'Nacital som obmedzenie: {lava} <= {prava}' )
        self.obmedzenia.append( (lava, prava) )

    def pridaj_instrukciu(self, instrukcia):
        try:
            lava, prava = [ x.strip() for x in instrukcia.split('->') ]
            lava = self.parsuj_mnozinu(lava)
            prava = self.parsuj_mnozinu(prava)
        except:
            assert False, f'Instrukcia "{instrukcia}" nie je syntakticky korektna.'
        if self.verbose:
            print( f'Nacital som instrukciu: {lava} -> {prava}' )
        self.instrukcie.append( (lava, prava) )

    def pouzi_instrukciu(self, stav, instrukcia):
        novy_stav = stav.copy()
        lava, prava = instrukcia
        for farba in lava:
            if farba not in novy_stav: return None
            if novy_stav[farba] < lava[farba]: return None
            novy_stav[farba] -= lava[farba]
        for farba in prava:
            if farba not in novy_stav:
                novy_stav[farba] = 0
            novy_stav[farba] += prava[farba]
        novy_stav = { x:novy_stav[x] for x in novy_stav if novy_stav[x] > 0 }
        return novy_stav

    def over_obmedzenia(self, stav):
        for lava, prava in self.obmedzenia:
            total = 0
            for farba in lava:
                if farba in stav:
                    total += stav[farba] * lava[farba]
            if total > prava:
                return False
        return True

    def simuluj(self, vstup):
        vstup = self.parsuj_mnozinu(vstup)
        if self.verbose:
            print( '-----' )
            print( f'Zacinam simulaciu na vstupe: {vstup}' )
        krokov = 0
        assert self.over_obmedzenia(vstup), f'Zaciatocne parametre "{vstup}" nesplnaju obmedzenia.'
        stav = vstup.copy()
        while True:
            if krokov == self.max_krokov:
                print(f'Na vstupe {vstup} program ani po {self.max_krokov} krokoch neskoncil.')
                return
            vykonal = False
            for instrukcia in self.instrukcie:
                novy_stav = self.pouzi_instrukciu(stav, instrukcia)
                if novy_stav is None: continue
                if not self.over_obmedzenia(novy_stav): continue
                vykonal = True
                stav = novy_stav
                krokov += 1
                if self.verbose:
                    print( f'  krok {krokov}: pouzil instrukciu {instrukcia[0]} -> {instrukcia[1]}.' )
                    print( f'  krok {krokov}: v Pokladnicke: {stav}' )
                break
            if not vykonal:
                break
        print(f'Na vstupe {vstup} po {krokov} krokoch dal program vystup {stav}.')


def starts_with(text, prefix):
    if len(prefix) > len(text): return False
    return text[:len(prefix)] == prefix

if __name__ == '__main__':
    try:
        verbose = any( x in sys.argv for x in [ '-v', '--verbose' ] )
        interpreter = Pokladnicka( verbose )
        for line in sys.stdin.readlines():
            line = line.strip()
            if line == '':
                continue
            if starts_with(line, '#'):
                continue
            if starts_with(line, 'OBMEDZENIE:'):
                interpreter.pridaj_obmedzenie( line[11:].strip() )
                continue
            if starts_with(line, 'VSTUP:'):
                interpreter.simuluj( line[6:].strip() )
                continue
            interpreter.pridaj_instrukciu( line.strip() )
    except Exception as error_message:
        print(error_message)



